/*
   Pouzite komponenty:
 *  * Arduino Nano ATmega328P
 *  * Senzor vlhkosti pudy 2x
 *  * DS18B20 vodeodolny teplomer
 *  * 433Mhz bezdratovy vysilac

    Pouzite knihovny:
 *  * Wifi.h, https://github.com/arduino-libraries/WiFi
 *  * HttpClient https://www.arduino.cc/reference/en/libraries/httpclient/
 *  * ArduinoJson.h https://github.com/bblanchon/ArduinoJson
 *  * DallasTemperature, https://github.com/milesburton/Arduino-Temperature-Control-Library
 *  * VirtualWire.h, http://www.airspayce.com/mikem/arduino/VirtualWire/

    Pri psani jsem castecne vychazel z tohoto zdroje: https://www.electronics-lab.com/project/using-433mhz-rf-transmitter-receiver-arduino/
*/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SH110X.h>
#include "AiEsp32RotaryEncoder.h"
#include "Arduino.h"
#include <MFRC522.h>
#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "server.h"

#define SCREEN_WIDTH 128     // OLED displej sirka v pixelech
#define SCREEN_HEIGHT 64     // OLED displej vyska v pixelech
#define SCREEN_ADDRESS 0x3C  // vyber typu displeje: 0x3D pro 128x64, 0x3C pro 128x32

/*-- přiřazení pinů --*/
#define OLED_RESET -1        // resetovací pin, nutny pro fungovani knihovny
#define OLEDC 22             // oled clock pin
#define OLEDD 21             // oled data pin
#define ROTARY_ENCODER_A_PIN 16
#define ROTARY_ENCODER_B_PIN 17
#define ROTARY_ENCODER_BUTTON_PIN 15
#define ROTARY_ENCODER_VCC_PIN -1
#define ROTARY_ENCODER_STEPS 4
#define QR Serial1
#define TX 26                // pin TX pro QR
#define RX 27                // pin RX pro QR
#define SS_PIN 5             // ESP32 pin GIOP5
#define RST_PIN 25           // ESP32 pin GIOP25
#define LED 12               // pin pro LED


/*-- inicializace knihovních objektů --*/
AiEsp32RotaryEncoder rotaryEncoder = AiEsp32RotaryEncoder(ROTARY_ENCODER_A_PIN, ROTARY_ENCODER_B_PIN, ROTARY_ENCODER_BUTTON_PIN, ROTARY_ENCODER_VCC_PIN, ROTARY_ENCODER_STEPS);
Adafruit_SH1106G display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);   //inicializace objektu display
MFRC522 rfid(SS_PIN, RST_PIN);  // instance třídy rfid

/*-- promenne --*/
String sRFID = "";    // textový řetězec s ID RFID tagu
String sQR = "";      // textový řetězec s ID čárového kódu
int selected = 0;     // vybraná proměnná
int entered = -1;     // hodnota na vstupu
int m2entered = -1;   // vybraná položka v submenu
int down = 1;         // pohyb směrem dolů
int up = 1;           // pohyb směrem nahoru
int enter = 1;        // potvrzení výběru
int back = 1;         // tlačítko zpět
int prev = 0;         // předchozí měnu
int act = 0;          // aktuální pozice rotačního enkodéru
int run = 0;          // iterační proměnná ukládající údaje o počtu běhů
int scanRes = -1;     // proměná ukládající úspěšnost skenování kódů


/*-- funkce --*/

// funkce blink rozbliká po zavolání stavovou LED diodu na pinu 12
void blink() {
  int wait = 200;
  digitalWrite(LED, HIGH);  // rozviti se LED
  delay(wait);              // ceka se po dobu "wait" v ms
  digitalWrite(LED, LOW);   // vypne se LED
  delay(wait);
}


// funkce fQR čte pomocí knihovny hodnoty čárových kódů 
void fQR() {
  if (QR.available()) /       // kontroluje, jestli jsou nová příchozí data v sériovém zásobníku
    while (QR.available()) {  // neustálé čtení bit po bitu, dokud zásobník není prázdný    
      char input = QR.read(); // čte 1 bite dat a ukládá je jako proměnnou input
      sQR += input;           // přičtení charu input do celého řetězce sQR
      delay(5);               // malá pauza
    }
    Serial.println(sQR);
    sQR.trim();               //odstrani whitespacy
    blink();
  }
}

// funkce fRFID čte pomocí knihovny hodnoty RFID tagů
void fRFID() {
  if (rfid.PICC_IsNewCardPresent()) {               // byla přiložena nová karta?
    if (rfid.PICC_ReadCardSerial()) {               // byla karta přčtena?      
      for (int i = 0; i < rfid.uid.size; i++) {     // tisk ID karty na seriový monitor v hex formátu
        sRFID += String(rfid.uid.uidByte[i], DEC);  // přičtení stringu ze čtečky RFID do celého řetězce sFRID
      }
      Serial.println(sRFID);
      sQR.trim();                                   // odstraní se whitespacy
      blink();                                      // volání blikání
      rfid.PICC_HaltA();                            // pozastaví čtení a zabrání opakování
      rfid.PCD_StopCrypto1();                       // zastaví se dekryptování
    }
  }
}

void rotary_loop() {
  //na seriove lince se nic nemeni, dokud neni otoceno enkoderem
  if (rotaryEncoder.encoderChanged()) {
    Serial.print("Value: ");
    act = rotaryEncoder.readEncoder();

    if (prev == act) {}
    if (prev < act) {
      up = LOW;
    }
    if (prev > act) {
      down = LOW;
    }
    prev = act;
    Serial.println(act);
  }
  if (rotaryEncoder.isEncoderButtonClicked()) {
    Serial.println("klik");
    enter = LOW;
  }
}

void IRAM_ATTR readEncoderISR() {
  rotaryEncoder.readEncoder_ISR();
}

void displaymenu(void) {
  rotary_loop();

  if (up == LOW && down == LOW) {};
  if (up == LOW) {
    if (selected != 2) {
      selected = selected + 1;
      delay(200);
      up = HIGH;
    }
  };
  if (down == LOW) {
    if (selected != 0) {
      selected = selected - 1;
      delay(200);
      down = HIGH;
    }
  };
  if (enter == LOW) {
    if (entered == -1) {
      entered = selected;
    } else if (entered == 0 || entered == 1 || entered == 2) {
      m2entered = selected;
      if (selected == 2) {
        entered = -1;
        selected = 0;
      }
    }

    enter = HIGH;
  };
  int menuItems = 3;
  const char *options[menuItems] = {        // statické pole obsahující položky menu
    " KNIHOVNA ",
    " DILNA ",
    " info "
  };

  const char *selectOptions[menuItems] = {
    " Pridat dalsi ",
    " Vyskladnit ",
    " zpet "
  };

  if (entered == -1) {
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.println(F("  -=Inventarizator=-"));
    display.println();
    for (int i = 0; i < menuItems; i++) {
      if (i == selected) {
        display.setTextColor(BLACK, WHITE);
        display.println(options[i]);
        display.println();
      } else if (i != selected) {
        display.setTextColor(WHITE);
        display.println(options[i]);
        display.println();
      }
    }
    sQR = "";
    sRFID = "";
    run = 0;
    m2entered = -1;
  } else if (entered == 0) {
    if ((sQR != "" && sRFID == "") || (sQR == "" && sRFID != "")) {
      delay(100);
      if (run == 0) {
        if (sQR != "") {
          scanRes = getQR(sQR);
        }
        if (sRFID != "") {
          scanRes = getRFID(sRFID);
        }
        selected = 0;
      }
      if (scanRes == 0) {
        display.clearDisplay();
        display.setTextSize(1);
        display.setCursor(0, 0);
        display.setTextColor(WHITE);
        display.println(name);
        display.setTextColor(BLACK, WHITE);
        display.print(F("Skladem:"));
        display.setTextColor(WHITE);
        display.print(" ");
        display.print(stock);
        display.println(F(" ks"));
        display.println();
        for (int i = 0; i < menuItems; i++) {
          if (i == selected) {
            display.setTextColor(BLACK, WHITE);
            display.println(selectOptions[i]);
          } else if (i != selected) {
            display.setTextColor(WHITE);
            display.println(selectOptions[i]);
          }
        }
        if (m2entered == 0) {
          String count = "1";
          if (sQR != "") {
            addQR(sQR, count);
            getQR(sQR);
          }
          if (sRFID != "") {
            addRFID(sRFID, count);
            getRFID(sRFID);
          }          
          m2entered = -1;
        } else if (m2entered == 1) {
          String count = "-1";
          if (sQR != "") {
            addQR(sQR, count);
            getQR(sQR);
          }
          if (sRFID != "") {
            addRFID(sRFID, count);
            getRFID(sRFID);
          }
          m2entered = -1;
        } else if (m2entered == 2) {
          entered = -1;
        }
        run++;
      }
      if (scanRes == 1) {
        if (run == 0) {
          if (sQR != "") {
            postQR(sQR);
          }
          if (sRFID != "") {
            postRFID(sRFID);
          }
        }
        selected = 2;
        display.clearDisplay();
        display.setTextSize(1);
        display.setTextColor(BLACK, WHITE);
        display.setCursor(0, 0);
        display.println(F(" Pridani knihy "));
        display.println();
        display.setTextColor(WHITE);
        display.print("Kniha s ");
        if (sQR != "") {
          display.println("ISBN:");
          display.println(sQR);
        }
        if (sRFID != "") {
          display.println("kodem:");
          display.println(sRFID);
        }        
        display.println("byla pridana do DB.");
        display.setTextSize(1);
        display.setTextColor(BLACK, WHITE);
        display.println();
        display.println(" zpet ");
        run++;
      }
    } else {
      selected = 2;
      display.clearDisplay();
      display.setTextSize(1);
      display.setTextColor(WHITE);
      display.setCursor(0, 0);
      display.println(F("KNIHOVNA"));
      display.println();
      display.setTextColor(WHITE);
      display.setTextSize(2);
      display.println("Cekam na\nscan!");
      display.setTextSize(1);
      display.setTextColor(BLACK, WHITE);
      display.println();
      display.println(" zpet ");
    }
  } else if (entered == 1) {
    if ((sQR != "" && sRFID == "") || (sQR == "" && sRFID != "")) {
      delay(100);
      if (run == 0) {
        if (sQR != "") {
          scanRes = WgetQR(sQR);
        }
        if (sRFID != "") {
          scanRes = WgetRFID(sRFID);
        }
        selected = 0;
      }
      if (scanRes == 0) {
        display.clearDisplay();
        display.setTextSize(1);
        display.setCursor(0, 0);
        display.setTextColor(WHITE);
        display.println(name);
        display.setTextColor(BLACK, WHITE);
        display.print(F("Skladem:"));
        display.setTextColor(WHITE);
        display.print(" ");
        display.print(stock);
        display.println(F(" ks"));
        display.println();
        for (int i = 0; i < menuItems; i++) {
          if (i == selected) {
            display.setTextColor(BLACK, WHITE);
            display.println(selectOptions[i]);
          } else if (i != selected) {
            display.setTextColor(WHITE);
            display.println(selectOptions[i]);
          }
        }
        if (m2entered == 0) {
          String count = "1";
          if (sQR != "") {
            WaddQR(sQR, count);
            WgetQR(sQR);
          }
          if (sRFID != "") {
            WaddRFID(sRFID, count);
            WgetRFID(sRFID);
          }          
          m2entered = -1;
        } else if (m2entered == 1) {
          String count = "-1";
          if (sQR != "") {
            WaddQR(sQR, count);
            WgetQR(sQR);
          }
          if (sRFID != "") {
            WaddRFID(sRFID, count);
            WgetRFID(sRFID);
          }
          m2entered = -1;
        } else if (m2entered == 2) {
          entered = -1;
        }
        run++;
      }
      if (scanRes == 1) {
        if (run == 0) {
          if (sQR != "") {
            WpostQR(sQR);
          }
          if (sRFID != "") {
            WpostRFID(sRFID);
          }
        }
        selected = 2;
        display.clearDisplay();
        display.setTextSize(1);
        display.setTextColor(BLACK, WHITE);
        display.setCursor(0, 0);
        display.println(F(" Pridani predmetu "));
        display.println();
        display.setTextColor(WHITE);
        display.print("Predmet s ");
        if (sQR != "") {
          display.println("kodem:");
          display.println(sQR);
        }
        if (sRFID != "") {
          display.println("kodem:");
          display.println(sRFID);
        }        
        display.println("byla pridana do DB.");
        display.setTextSize(1);
        display.setTextColor(BLACK, WHITE);
        display.println();
        display.println(" zpet ");
        run++;
      }
    } else {
      selected = 2;
      display.clearDisplay();
      display.setTextSize(1);
      display.setTextColor(WHITE);
      display.setCursor(0, 0);
      display.println(F("DILNA"));
      display.println();
      display.setTextColor(WHITE);
      display.setTextSize(2);
      display.println("Cekam na\nscan!");
      display.setTextSize(1);
      display.setTextColor(BLACK, WHITE);
      display.println();
      display.println(" zpet ");
    }
  } else if (entered == 2) {
    selected = 2;
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.println(F("SITOVE INFO"));
    display.println();
    display.setTextColor(BLACK, WHITE);
    display.print(" IP:");
    display.setTextColor(WHITE);
    display.print(" ");
    display.println(WiFi.localIP());
    display.setTextColor(BLACK, WHITE);
    display.print(" SSID:");
    display.setTextColor(WHITE);
    display.print(" ");
    display.println(WIFI_SSID);
    display.println();
    display.println();
    display.setTextColor(BLACK, WHITE);
    display.println(" zpet ");
  }
  display.display();
}

void setup() {
  Serial.begin(115200);

  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  Serial.println("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());


  SPI.begin();                         // inicializace SPI sbernice
  rfid.PCD_Init();                     // inicializace MFRC522 ctecky karet
  QR.begin(9600, SERIAL_8N1, TX, RX);  // Definování seriového portu pro QR čtečku GM65 TX=26,RX=27
  //viz dokumentace QR, 8 databitu, jeden stop..8N1 https://github.com/G6EJD/ESP32-Using-Hardware-Serial-Ports/blob/master/ESP32_Using_Serial2.ino
  pinMode(LED, OUTPUT);
  if (!display.begin(SCREEN_ADDRESS, true)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;)
      ;
  }
  display.clearDisplay();              //nakresli se jeden bily bod
  display.drawPixel(10, 10, WHITE);
  display.display();
  delay(2000);                        //pauza 2s

  rotaryEncoder.begin();              //inicializace rotacniho enkoderu
  rotaryEncoder.setup(readEncoderISR);

  //set boundaries and if values should cycle or not
  //in this example we will set possible values between 0 and 1000;
  bool circleValues = false;
  rotaryEncoder.setBoundaries(-50, 50, circleValues);  //minValue, maxValue, circleValues true|false (when max go to min and vice versa)
  rotaryEncoder.disableAcceleration();                 //acceleration is now enabled by default - disable if you dont need it
}

void loop() {
  fQR();
  fRFID();
  displaymenu();
  delay(50);
}
